{#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
-#}
<!DOCTYPE HTML>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
    <title>Jasmine Spec Runner</title>
    {% bundle "jasmine.js", "require.js", "jasmine.css" %}
    {%- include "base/stylesheets.html" %}
    {% include "base/scripts.html" %}
    <script type="text/javascript">
    // Test files are loaded using RequireJS. Each test file in turns uses RequireJS
    // to load the JS files they test.
    require(
        ['jasmine-boot',
        {% for m in modules %}'{{m}}'{%if not loop.last %}, {% endif %}{% endfor %}
        ],
        function (executeTests) {
            // Jasmine runs tests on window.onload() by default, thus before
            // require.js has had a chance to load them. Thus we use a custom
            // boot.js file which renames window.onload to window.executeTests and
            // calls the executeTests() once the test modules have been loaded.
            executeTests();
        }
    )
    </script>
</head>
<body>
</body>
</html>
